-------------------------------------------------------------------------------
-- Creates a FP representation of a number from its sign, exponent and mantissa
--
-- Generics:
--   - wE [positive] : exponent bit-width
--   - wF [positive] : fraction bit-width
--
-- Ports:
--   - s  [in]  : sign
--   - e  [in]  : exponent
--   - m  [in]  : mantissa
--   - fp [out] : formated FP representation
--
-- Tests for zeros and infinities.
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity fp_format is
  generic ( wE : positive;
            wF : positive );
  port ( s  : in  std_logic;
         e  : in  std_logic_vector(wE downto 0);
         m  : in  std_logic_vector(wF downto 0);
         fp : out std_logic_vector(wE+wF+2 downto 0) );
end entity;

architecture arch of fp_format is
  signal eMax  : std_logic;
  signal eMin  : std_logic;
  signal eTest : std_logic_vector(1 downto 0);
begin -------------------------------------------------------------------------

  eMax <= '1' when e(wE-1 downto 0) = (wE-1 downto 0 => '1') else
          '0';

  eMin <= '1' when e(wE-1 downto 0) = (wE-1 downto 0 => '0') else
          '0';

  eTest(1) <= (eMax and not e(wE)) or (e(wE) and not e(wE-1));
  eTest(0) <= (eMin and not e(wE)) or (e(wE) and e(wE-1)) or not m(wF);

  fp(wE+wF+2 downto wE+wF+1) <= "00" when eTest = "01" else
                                "10" when eTest = "10" else
                                "01";

  fp(wE+wF downto 0) <= s & e(wE-1 downto 0) & m(wF-1 downto 0);

end architecture; -------------------------------------------------------------
