-------------------------------------------------------------------------------
-- Internal package for sine and cosine operators
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

package pkg_fp_sin_cos is

  -----------------------------------------------------------------------------
  -- 
  -----------------------------------------------------------------------------

  function fp_sin_cos_4oPi_g ( wE, wF : positive ) return positive;

  -----------------------------------------------------------------------------
  -- 
  -----------------------------------------------------------------------------

  function fp_sin_cos_4oPi ( wE, wF : positive ) return std_logic_vector;

  -----------------------------------------------------------------------------
  -- 
  -----------------------------------------------------------------------------

  function fp_sin_cos_Pio4 ( wF : positive ) return std_logic_vector;

  -----------------------------------------------------------------------------
  -- 
  -----------------------------------------------------------------------------

  component fp_sin_cos_fsin is
    generic ( wF : positive );
    port ( x : in  std_logic_vector(wF+1 downto 0);
           r : out std_logic_vector(wF+2 downto 0) );
  end component;

  -----------------------------------------------------------------------------
  -- 
  -----------------------------------------------------------------------------

  component fp_sin_cos_fcos is
    generic ( wF : positive );
    port ( x : in  std_logic_vector(wF+1 downto 0);
           r : out std_logic_vector(wF+2 downto 0) );
  end component;
  
  -----------------------------------------------------------------------------
  -- 
  -----------------------------------------------------------------------------

  component fp_sin_cos_fsin_clk is
    generic ( wF : positive );
    port ( x : in  std_logic_vector(wF+1 downto 0);
           r : out std_logic_vector(wF+2 downto 0);
           clk : in  std_logic );
  end component;

  -----------------------------------------------------------------------------
  -- 
  -----------------------------------------------------------------------------

  component fp_sin_cos_fcos_clk is
    generic ( wF : positive );
    port ( x : in  std_logic_vector(wF+1 downto 0);
           r : out std_logic_vector(wF+2 downto 0);
           clk : in  std_logic );
  end component;

  -----------------------------------------------------------------------------
  -- 
  -----------------------------------------------------------------------------

  component fp_sin_cos_fsin2 is
    generic ( wF : positive );
    port ( x : in  std_logic_vector(wF+2 downto 0);
           r : out std_logic_vector(wF+4 downto 0) );
  end component;

  -----------------------------------------------------------------------------
  -- 
  -----------------------------------------------------------------------------

  component fp_sin_cos_sin is
    generic ( wF : positive );
    port ( x : in  std_logic_vector(wF+1 downto 0);
           r : out std_logic_vector(wF+2 downto 0) );
  end component;

  -----------------------------------------------------------------------------
  -- 
  -----------------------------------------------------------------------------

  component fp_sin_cos_cos is
    generic ( wF : positive );
    port ( x : in  std_logic_vector(wF+1 downto 0);
           r : out std_logic_vector(wF+2 downto 0) );
  end component;

end package;

-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_unsigned.all;

package body pkg_fp_sin_cos is

  -----------------------------------------------------------------------------
  -- 
  -----------------------------------------------------------------------------

  type int_matrix is array (positive range <>, positive range <>) of positive;
  
  constant cst_4oPi_g : int_matrix(5 to 8, 10 to 23) :=
    -- 10  11  12  13  14  15  16  17  18  19  20  21  22  23     wF/wE
    ( (15,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 ),   -- 5
      ( 1,  1,  1, 20,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1 ),   -- 6
      ( 1,  1,  1,  1,  1,  1, 24,  1,  1,  1, 25,  1,  1,  1 ),   -- 7
      ( 1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1, 28 ) ); -- 8

  function fp_sin_cos_4oPi_g ( wE, wF : positive ) return positive is
  begin
    return cst_4oPi_g(8, 23);
  end function;

  -----------------------------------------------------------------------------
  -- 
  -----------------------------------------------------------------------------

  constant cst_4oPi : std_logic_vector(23+3 + 23+127+28+6 downto 0) :=
    b"000000000000000000000000001_01000101111100110000011011" & --   26 to  -26
     "011100100111001000100000101010010100111111100001001110" & --  -27 to  -80
     "101011111010001111101010011010011011101110000001101101" & --  -81 to -134
     "10110001010010101100110010011110001000011100100000";      -- -135 to -184

  function fp_sin_cos_4oPi ( wE, wF : positive ) return std_logic_vector is
    constant g : positive := fp_sin_cos_4oPi_g(wE, wF);
  begin
    return cst_4oPi(184 + wF+3 downto 184 - (wF+2**(wE-1)-1+g+6));
  end function;

  -----------------------------------------------------------------------------
  -- 
  -----------------------------------------------------------------------------

  constant cst_Pio4 : std_logic_vector(49 downto 0) :=
    b"11001001000011111101101010100010001000010110100011";      --   -1 to  -50

  function fp_sin_cos_Pio4 ( wF : positive ) return std_logic_vector is
  begin
    return cst_Pio4(49 downto 49 - (wF+4)) + cst_Pio4(49 - (wF+5));
  end function;

end package body;
